@@ -1,13 +1,15 @@
-#' @title checkNEvalPts - Check number of numeric integration intervals
+#' @title Check number of numeric integration intervals
 #' 
 #' @description
 #' Check that number of integration intervals is odd and 
-#' sufficiently large. 
+#' sufficiently large. Plus, compute and store the Simpson's Composite
+#' rule coefficients. 
 #' 
 #' @param nEvalPts An integer to check.
 #' 
 #' @return The first element of nEvalPts is returned if it is acceptable.
-#' If nEvalPts is not acceptable, an error is thrown.
+#' If nEvalPts is not acceptable, an error is thrown. Simpson's composite
+#' coefficients are store in options()
 #' 
 checkNEvalPts <- function(nEvalPts){
   
@@ -33,5 +35,9 @@ checkNEvalPts <- function(nEvalPts){
                     ))
   }
   
+  # Store Simpson coefficients in options() to speed calculations a little
+  intCoefs <- simpsonCoefs( nEvalPts )
+  options("Rdistance_intCoefs" = intCoefs)
+  
   invisible(nEvalPts)
-}
\ No newline at end of file
+}
