@@ -1,4 +1,4 @@
-#' @title dE.multi - Estimate multiple-observer line-transect distance functions
+#' @title Estimate multiple-observer line-transect distance functions
 #' 
 #' @description Fits a detection function to off-transect 
 #' distances collected by multiple observers. 
@@ -13,11 +13,11 @@
 dE.multi <- function( data
                          , formula
                          , likelihood = "halfnorm"
-                         , w.lo = units::set_units(0,"m")
+                         , w.lo = setUnits(0,"m")
                          , w.hi = NULL
                          , expansions = 0
                          , series = "cosine"
-                         , x.scl = units::set_units(0,"m")
+                         , x.scl = setUnits(0,"m")
                          , g.x.scl = 1
                          , warn = TRUE
                          , outputUnits = NULL){
@@ -30,4 +30,4 @@ dE.multi <- function( data
           versions >2.x, so they have been disabled for the time being.
           Contact the Rdistance authors if you need double observer analyses.")
 
-}
\ No newline at end of file
+}
